==> fred-verifier-0.4.1.tar.gz: fred-verifier-0.4.1/verifier/client.py <==
"""Verification service client."""
from datetime import timedelta
from typing import Any, Dict, List, Optional, Tuple, Union, cast

import fred_api.verification.service_verification_grpc_pb2_grpc
from fred_api.verification.common_types_pb2 import (
    ContactDoesNotExist as ContactDoesNotExistMessage,
    Event as EventMessage,
    IncorrectStatus as IncorrectStatusMessage,
    InvalidData as InvalidDataMessage,
    MethodNotApplicable as MethodNotApplicableMessage,
    PlaceAddress,
    VerificationDoesNotExist as VerificationDoesNotExistMessage,
    VerificationInfo,
    VerificationStatus as VerificationEnum,
)
from fred_api.verification.pagination_pb2 import PaginationReply
from fred_api.verification.service_verification_grpc_pb2 import (
    ApproveFutileRequest,
    CancelRequest,
    CreateRequest,
    DelayStatusRequest,
    GetContactRepresentativeRequest,
    GetMethodsRequest,
    GetRequest,
    ListRequest,
    ListVerificationResultsRequest,
    RequestManualCheckRequest,
    ResendMessageRequest,
    ResolveManualCheckRequest,
    SetContactRepresentativeRequest,
    UseMethodRequest,
    VerifyContactDataRequest,
    VerifyTokenRequest,
)
from frgal import GrpcDecoder
from frgal.aio import AsyncGrpcClient
from regal import Address, ContactHistoryId, ContactId

from .common import (
    ContactData,
    ContactRepresentative,
    Event,
    PaginatedList,
    Pagination,
    Verification,
    VerificationId,
    VerificationResult,
    VerificationStatus,
)
from .exceptions import (
    ContactDoesNotExist,
    IncorrectStatus,
    InvalidData,
    MethodNotApplicable,
    VerificationDoesNotExist,
)


class VerificationDecoder(GrpcDecoder):
    """Registrar service decoder."""

    decode_unset_messages = False
    decode_empty_string_none = False

    def __init__(self) -> None:
        super().__init__()
        self.set_decoder(EventMessage, self._decode_event)
        self.set_decoder(VerificationInfo, self._decode_verification)
        self.set_decoder(PaginationReply, self._decode_pagination)
        self.set_decoder(PlaceAddress, self._decode_address)

        self.set_exception_decoder(ContactDoesNotExistMessage, ContactDoesNotExist)
        self.set_exception_decoder(
            VerificationDoesNotExistMessage, VerificationDoesNotExist
        )
        self.set_exception_decoder(MethodNotApplicableMessage, MethodNotApplicable)
        self.set_exception_decoder(InvalidDataMessage, InvalidData)
        self.set_exception_decoder(IncorrectStatusMessage, IncorrectStatus)

    def _decode_event(self, value: EventMessage) -> Event:
        dict_value = self._decode_message(value)
        if dict_value["details"] is None:
            dict_value.pop("details")
        return Event(**dict_value)

    def _decode_verification(self, value: VerificationInfo) -> Verification:
        dict_value = self._decode_message(value)
        dict_value.update({"status": VerificationEnum.Name(value.status)})
        return Verification(**dict_value)

    def _decode_pagination(self, value: PaginationReply) -> Pagination:
        return Pagination(**self._decode_message(value))

    def _decode_address(self, value: PlaceAddress) -> Address:
        return Address(**self._decode_message(value))


class VerificationClient(AsyncGrpcClient):
    """Verification service gRPC client."""

    service = "Verification"
    decoder_cls = VerificationDecoder
    grpc_modules = (fred_api.verification.service_verification_grpc_pb2_grpc,)

    async def create(self, contact_id: ContactId) -> Verification:
        """Create verification for contact.

        Args:
            contact_id: Contact id.

        Returns:
            Verification info.

        Raises:
            ContactDoesNotExist: If contact does not exist.
        """
        request = CreateRequest()
        request.contact_id.value = contact_id
        return cast(Verification, await self.call(self.service, "create", request))

    async def cancel(self, verification_id: VerificationId) -> None:
        """Cancel verification.

        Args:
            verification_id: Verification id.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            IncorrectStatus: If verification is already closed.
        """
        request = CancelRequest()
        request.verification_id.value = verification_id
        await self.call(self.service, "cancel", request)

    async def get(
        self, verification_id: VerificationId, *, access_token: Optional[str] = None
    ) -> Verification:
        """Get verification info.

        Args:
            verification_id: Verification id.
            access_token: Access token to be verified.

        Returns:
            Verification info.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = GetRequest()
        request.verification_id.value = verification_id
        if access_token:
            request.access_token = access_token
        return cast(Verification, await self.call(self.service, "get", request))

    async def list(
        self,
        contact_id: Optional[ContactId] = None,
        include_closed: bool = False,
        *,
        page_token: Optional[str] = None,
        page_size: Optional[int] = None,
    ) -> PaginatedList[Verification]:
        """List verifications.

        Args:
            contact_id: When set, filter verifications by contact_id.
            include_closed: If True, include closed verifications in the result.
            page_token: Page token for pagination.
            page_size: Page size for pagination.

        Returns:
            Paginated list of verifications.
        """
        request = ListRequest()
        if contact_id is not None:
            request.contact_id.value = contact_id
        request.include_closed = include_closed
        if page_token is not None:
            request.pagination.page_token = page_token
        if page_size is not None:
            request.pagination.page_size = page_size

        result: PaginatedList[Verification] = PaginatedList(results=[])
        async for message in self.call_stream(
            self.service, "list", request
        ):  # pragma: no cover
            if message:
                if result.pagination is None:
                    result.pagination = message.get("pagination")
                result.results.extend(message.get("verifications", []))

        return result

    async def get_methods(self, verification_id: VerificationId) -> List[str]:
        """Get available methods for verification.

        Args:
            verification_id: Verification id.

        Returns:
            List of available methods.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = GetMethodsRequest()
        request.verification_id.value = verification_id
        result = await self.call(self.service, "get_methods", request)
        return cast(List[str], result)

    async def use_method(
        self, verification_id: VerificationId, method: str
    ) -> Verification:
        """Use chosen method for verification.

        Args:
            verification_id: Verification id.
            method: Chosen method name.

        Returns:
            Verification info.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            MethodNotAvailable: If method is not available for verification.
            IncorrectStatus: If verification is in incorrect state.
        """
        request = UseMethodRequest()
        request.verification_id.value = verification_id
        request.method = method
        result = await self.call(self.service, "use_method", request)
        return cast(Verification, result)

    async def verify_token(self, verification_id: VerificationId, token: str) -> bool:
        """Verify token for a verification.

        Args:
            verification_id: Verification id.
            token: Token.

        Returns:
            Boolean indicating whether token is valid for the verification.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            IncorrectStatus: If verification is already closed.
        """
        request = VerifyTokenRequest()
        request.verification_id.value = verification_id
        request.token = token
        return cast(bool, await self.call(self.service, "verify_token", request))

    async def verify_contact_data(
        self, verification_id: VerificationId, method: str, data: ContactData
    ) -> Tuple[Verification, VerificationResult]:
        """Verify contact data.

        Args:
            verification_id: Verification id.
            method: Verification method.
            data: Contact data to verify.

        Returns:
            Updated verification and verification result.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            MethodNotAvailable: If method is not available for verification.
            InvalidData: If data format is invalid.
            IncorrectStatus: If verification is in incorrect state.
        """
        request = VerifyContactDataRequest()
        request.verification_id.value = verification_id
        request.method = method
        if data.name is not None:
            request.data.name = data.name
        if data.organization is not None:
            request.data.organization = data.organization
        if data.birthdate is not None:
            request.data.birthdate = data.birthdate.isoformat()
        if data.company_registration_number is not None:
            request.data.company_registration_number = data.company_registration_number
        if data.address is not None:
            request.data.address.street[:] = data.address.street
            request.data.address.city = data.address.city
            request.data.address.state_or_province = data.address.state_or_province
            request.data.address.postal_code = data.address.postal_code
            request.data.address.country_code = data.address.country_code

        result = await self.call(self.service, "verify_contact_data", request)
        return (
            result.get("verification"),
            VerificationResult(**result.get("result")),
        )

    async def list_verification_results(
        self, verification_id: VerificationId
    ) -> List[VerificationResult]:
        """Get list of contact data verifications.

        Args:
            verification_id: Verification id.

        Returns:
            List of contact data verifications.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = ListVerificationResultsRequest()
        request.verification_id.value = verification_id

        verifications = await self.call(
            self.service, "list_verification_results", request
        )
        return [VerificationResult(**result) for result in verifications]

    async def request_manual_check(self, verification_id: VerificationId) -> None:
        """Request manual data check.

        Args:
            verification_id: Verification id.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            IncorrectStatus: If verification is in incorrect state.
        """
        request = RequestManualCheckRequest()
        request.verification_id.value = verification_id

        await self.call(self.service, "request_manual_check", request)

    async def resolve_manual_check(
        self,
        verification_id: VerificationId,
        *,
        verified: bool,
        contact_data: ContactData,
        contact_history_id: ContactHistoryId,
    ) -> None:
        """Resolve manual check of contact data.

        Args:
            verification_id: Verification id.
            verified: Whether provided data are verified or not.
            contact_data: Contact data being manually checked.
            contact_history_id: Contact history id against which were `contact_data` checked.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            InvalidData: If data format is invalid.
            IncorrectStatus: If verification is in incorrect state.
        """
        request = ResolveManualCheckRequest()
        request.verification_id.value = verification_id
        request.verified = verified
        request.data.CopyFrom(contact_data.to_protobuf())
        request.contact_history_id.value = contact_history_id

        await self.call(self.service, "resolve_manual_check", request)

    async def resend_message(
        self,
        verification_id: VerificationId,
        *,
        message_type: str,
        recipient: Optional[Dict[str, Any]] = None,
    ) -> None:
        """Resend message.

        Args:
            verification_id: Verification id.
            message_type: Message type.
            recipient: Custom recipient if required. Message-specific dict.
                If not provided, backend uses current contact address.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            IncorrectStatus: If verification is in incorrect state.
        """
        request = ResendMessageRequest()
        request.verification_id.value = verification_id
        request.message_type = message_type
        request.recipient.update(recipient or {})

        await self.call(self.service, "resend_message", request)

    async def approve_futile(self, verification_id: VerificationId) -> None:
        """Approve verification futility.

        This may lead to irreversible changes like domain deletion.

        Args:
            verification_id: Verification id.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            IncorrectStatus: If verification is in incorrect state.
        """
        request = ApproveFutileRequest()
        request.verification_id.value = verification_id
        await self.call(self.service, "approve_futile", request)

    async def delay_status(
        self,
        verification_id: VerificationId,
        *,
        status: VerificationStatus,
        delay: Union[timedelta, int],
    ) -> None:
        """Set status delay.

        Args:
            verification_id: Verification id.
            status: Verification status that should be delayed.
            delay: Delay. Can be timedelta, or int with number of seconds.
                Negative values are possible to shorten status timeout.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            IncorrectStatus: If verification is in incorrect state.
        """
        if isinstance(delay, timedelta):
            delay = int(delay.total_seconds())

        request = DelayStatusRequest()
        request.verification_id.value = verification_id
        request.status = VerificationEnum.Value(status)
        request.delay = delay
        await self.call(self.service, "delay_status", request)

    async def get_contact_representative(
        self, verification_id: VerificationId
    ) -> ContactRepresentative:
        """Get contact representative.

        Args:
            verification_id: Verification id.

        Returns:
            Contact representative.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = GetContactRepresentativeRequest()
        request.verification_id.value = verification_id
        return ContactRepresentative(
            **await self.call(self.service, "get_contact_representative", request)
        )

    async def set_contact_representative(
        self, verification_id: VerificationId, representative: ContactRepresentative
    ) -> None:
        """Set contact representative.

        Args:
            verification_id: Verification id.
            representative: Contact representative.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            IncorrectStatus: If verification is in incorrect state.
        """
        request = SetContactRepresentativeRequest()
        request.verification_id.value = verification_id
        request.organization = representative.organization
        request.address.street[:] = representative.address.street
        request.address.city = representative.address.city
        request.address.state_or_province = representative.address.state_or_province
        request.address.postal_code = representative.address.postal_code
        request.address.country_code = representative.address.country_code
        await self.call(self.service, "set_contact_representative", request)

==> fred-verifier-0.4.2.tar.gz: fred-verifier-0.4.2/verifier/client.py <==
"""Verification service client."""
from datetime import timedelta
from typing import Any, Dict, List, Optional, Tuple, Union, cast

import fred_api.verification.service_verification_grpc_pb2_grpc
from fred_api.verification.common_types_pb2 import (
    ContactDoesNotExist as ContactDoesNotExistMessage,
    Event as EventMessage,
    IncorrectStatus as IncorrectStatusMessage,
    InvalidData as InvalidDataMessage,
    MethodNotApplicable as MethodNotApplicableMessage,
    PlaceAddress,
    VerificationDoesNotExist as VerificationDoesNotExistMessage,
    VerificationInfo,
    VerificationStatus as VerificationEnum,
)
from fred_api.verification.pagination_pb2 import PaginationReply
from fred_api.verification.service_verification_grpc_pb2 import (
    ApproveFutileRequest,
    CancelRequest,
    CreateRequest,
    DelayStatusRequest,
    GetContactRepresentativeRequest,
    GetMethodsRequest,
    GetRequest,
    ListRequest,
    ListVerificationResultsRequest,
    RequestManualCheckRequest,
    ResendMessageRequest,
    ResolveManualCheckRequest,
    SetContactRepresentativeRequest,
    UseMethodRequest,
    VerifyContactDataRequest,
    VerifyTokenRequest,
)
from frgal import GrpcDecoder
from frgal.aio import AsyncGrpcClient
from regal import Address, ContactHistoryId, ContactId

from .common import (
    ContactData,
    ContactRepresentative,
    Event,
    PaginatedList,
    Pagination,
    Verification,
    VerificationId,
    VerificationResult,
    VerificationStatus,
)
from .exceptions import (
    ContactDoesNotExist,
    IncorrectStatus,
    InvalidData,
    MethodNotApplicable,
    VerificationDoesNotExist,
)


class VerificationDecoder(GrpcDecoder):
    """Registrar service decoder."""

    decode_unset_messages = False
    decode_empty_string_none = False

    def __init__(self) -> None:
        super().__init__()
        self.set_decoder(EventMessage, self._decode_event)
        self.set_decoder(VerificationInfo, self._decode_verification)
        self.set_decoder(PaginationReply, self._decode_pagination)
        self.set_decoder(PlaceAddress, self._decode_address)

        self.set_exception_decoder(ContactDoesNotExistMessage, ContactDoesNotExist)
        self.set_exception_decoder(
            VerificationDoesNotExistMessage, VerificationDoesNotExist
        )
        self.set_exception_decoder(MethodNotApplicableMessage, MethodNotApplicable)
        self.set_exception_decoder(InvalidDataMessage, InvalidData)
        self.set_exception_decoder(IncorrectStatusMessage, IncorrectStatus)

    def _decode_event(self, value: EventMessage) -> Event:
        dict_value = self._decode_message(value)
        if dict_value["details"] is None:
            dict_value.pop("details")
        return Event(**dict_value)

    def _decode_verification(self, value: VerificationInfo) -> Verification:
        dict_value = self._decode_message(value)
        dict_value.update({"status": VerificationEnum.Name(value.status)})
        return Verification(**dict_value)

    def _decode_pagination(self, value: PaginationReply) -> Pagination:
        return Pagination(**self._decode_message(value))

    def _decode_address(self, value: PlaceAddress) -> Address:
        return Address(**self._decode_message(value))


class VerificationClient(AsyncGrpcClient):
    """Verification service gRPC client."""

    service = "Verification"
    decoder_cls = VerificationDecoder
    grpc_modules = (fred_api.verification.service_verification_grpc_pb2_grpc,)

    async def create(self, contact_id: ContactId) -> Verification:
        """Create verification for contact.

        Args:
            contact_id: Contact id.

        Returns:
            Verification info.

        Raises:
            ContactDoesNotExist: If contact does not exist.
        """
        request = CreateRequest()
        request.contact_id.value = contact_id
        return cast(Verification, await self.call(self.service, "create", request))

    async def cancel(self, verification_id: VerificationId) -> None:
        """Cancel verification.

        Args:
            verification_id: Verification id.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            IncorrectStatus: If verification is already closed.
        """
        request = CancelRequest()
        request.verification_id.value = verification_id
        await self.call(self.service, "cancel", request)

    async def get(
        self, verification_id: VerificationId, *, access_token: Optional[str] = None
    ) -> Verification:
        """Get verification info.

        Args:
            verification_id: Verification id.
            access_token: Access token to be verified.

        Returns:
            Verification info.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = GetRequest()
        request.verification_id.value = verification_id
        if access_token:
            request.access_token = access_token
        return cast(Verification, await self.call(self.service, "get", request))

    async def list(
        self,
        contact_id: Optional[ContactId] = None,
        include_closed: bool = False,
        *,
        page_token: Optional[str] = None,
        page_size: Optional[int] = None,
    ) -> PaginatedList[Verification]:
        """List verifications.

        Args:
            contact_id: When set, filter verifications by contact_id.
            include_closed: If True, include closed verifications in the result.
            page_token: Page token for pagination.
            page_size: Page size for pagination.

        Returns:
            Paginated list of verifications.
        """
        request = ListRequest()
        if contact_id is not None:
            request.contact_id.value = contact_id
        request.include_closed = include_closed
        if page_token is not None:
            request.pagination.page_token = page_token
        if page_size is not None:
            request.pagination.page_size = page_size

        result: PaginatedList[Verification] = PaginatedList(results=[])
        async for message in self.call_stream(
            self.service, "list", request
        ):  # pragma: no cover
            if message:
                if result.pagination is None:
                    result.pagination = message.get("pagination")
                result.results.extend(message.get("verifications", []))

        return result

    async def get_methods(self, verification_id: VerificationId) -> List[str]:
        """Get available methods for verification.

        Args:
            verification_id: Verification id.

        Returns:
            List of available methods.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = GetMethodsRequest()
        request.verification_id.value = verification_id
        result = await self.call(self.service, "get_methods", request)
        return cast(List[str], result)

    async def use_method(
        self, verification_id: VerificationId, method: str
    ) -> Verification:
        """Use chosen method for verification.

        Args:
            verification_id: Verification id.
            method: Chosen method name.

        Returns:
            Verification info.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            MethodNotAvailable: If method is not available for verification.
            IncorrectStatus: If verification is in incorrect state.
        """
        request = UseMethodRequest()
        request.verification_id.value = verification_id
        request.method = method
        result = await self.call(self.service, "use_method", request)
        return cast(Verification, result)

    async def verify_token(self, verification_id: VerificationId, token: str) -> bool:
        """Verify token for a verification.

        Args:
            verification_id: Verification id.
            token: Token.

        Returns:
            Boolean indicating whether token is valid for the verification.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            IncorrectStatus: If verification is already closed.
        """
        request = VerifyTokenRequest()
        request.verification_id.value = verification_id
        request.token = token
        return cast(bool, await self.call(self.service, "verify_token", request))

    async def verify_contact_data(
        self, verification_id: VerificationId, method: str, data: ContactData
    ) -> Tuple[Verification, VerificationResult]:
        """Verify contact data.

        Args:
            verification_id: Verification id.
            method: Verification method.
            data: Contact data to verify.

        Returns:
            Updated verification and verification result.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            MethodNotAvailable: If method is not available for verification.
            InvalidData: If data format is invalid.
            IncorrectStatus: If verification is in incorrect state.
        """
        request = VerifyContactDataRequest()
        request.verification_id.value = verification_id
        request.method = method
        if data.name is not None:
            request.data.name = data.name
        if data.organization is not None:
            request.data.organization = data.organization
        if data.birthdate is not None:
            request.data.birthdate = data.birthdate.isoformat()
        if data.company_registration_number is not None:
            request.data.company_registration_number = data.company_registration_number
        if data.address is not None:
            request.data.address.street[:] = data.address.street
            request.data.address.city = data.address.city
            request.data.address.state_or_province = data.address.state_or_province
            request.data.address.postal_code = data.address.postal_code
            request.data.address.country_code = data.address.country_code

        result = await self.call(self.service, "verify_contact_data", request)
        return (
            result.get("verification"),
            VerificationResult(**result.get("result")),
        )

    async def list_verification_results(
        self, verification_id: VerificationId
    ) -> List[VerificationResult]:
        """Get list of contact data verifications.

        Args:
            verification_id: Verification id.

        Returns:
            List of contact data verifications.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = ListVerificationResultsRequest()
        request.verification_id.value = verification_id

        verifications = await self.call(
            self.service, "list_verification_results", request
        )
        return [VerificationResult(**result) for result in verifications or []]

    async def request_manual_check(self, verification_id: VerificationId) -> None:
        """Request manual data check.

        Args:
            verification_id: Verification id.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            IncorrectStatus: If verification is in incorrect state.
        """
        request = RequestManualCheckRequest()
        request.verification_id.value = verification_id

        await self.call(self.service, "request_manual_check", request)

    async def resolve_manual_check(
        self,
        verification_id: VerificationId,
        *,
        verified: bool,
        contact_data: ContactData,
        contact_history_id: ContactHistoryId,
    ) -> None:
        """Resolve manual check of contact data.

        Args:
            verification_id: Verification id.
            verified: Whether provided data are verified or not.
            contact_data: Contact data being manually checked.
            contact_history_id: Contact history id against which were `contact_data` checked.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            InvalidData: If data format is invalid.
            IncorrectStatus: If verification is in incorrect state.
        """
        request = ResolveManualCheckRequest()
        request.verification_id.value = verification_id
        request.verified = verified
        request.data.CopyFrom(contact_data.to_protobuf())
        request.contact_history_id.value = contact_history_id

        await self.call(self.service, "resolve_manual_check", request)

    async def resend_message(
        self,
        verification_id: VerificationId,
        *,
        message_type: str,
        recipient: Optional[Dict[str, Any]] = None,
    ) -> None:
        """Resend message.

        Args:
            verification_id: Verification id.
            message_type: Message type.
            recipient: Custom recipient if required. Message-specific dict.
                If not provided, backend uses current contact address.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            IncorrectStatus: If verification is in incorrect state.
        """
        request = ResendMessageRequest()
        request.verification_id.value = verification_id
        request.message_type = message_type
        request.recipient.update(recipient or {})

        await self.call(self.service, "resend_message", request)

    async def approve_futile(self, verification_id: VerificationId) -> None:
        """Approve verification futility.

        This may lead to irreversible changes like domain deletion.

        Args:
            verification_id: Verification id.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            IncorrectStatus: If verification is in incorrect state.
        """
        request = ApproveFutileRequest()
        request.verification_id.value = verification_id
        await self.call(self.service, "approve_futile", request)

    async def delay_status(
        self,
        verification_id: VerificationId,
        *,
        status: VerificationStatus,
        delay: Union[timedelta, int],
    ) -> None:
        """Set status delay.

        Args:
            verification_id: Verification id.
            status: Verification status that should be delayed.
            delay: Delay. Can be timedelta, or int with number of seconds.
                Negative values are possible to shorten status timeout.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            IncorrectStatus: If verification is in incorrect state.
        """
        if isinstance(delay, timedelta):
            delay = int(delay.total_seconds())

        request = DelayStatusRequest()
        request.verification_id.value = verification_id
        request.status = VerificationEnum.Value(status)
        request.delay = delay
        await self.call(self.service, "delay_status", request)

    async def get_contact_representative(
        self, verification_id: VerificationId
    ) -> ContactRepresentative:
        """Get contact representative.

        Args:
            verification_id: Verification id.

        Returns:
            Contact representative.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
        """
        request = GetContactRepresentativeRequest()
        request.verification_id.value = verification_id
        return ContactRepresentative(
            **await self.call(self.service, "get_contact_representative", request)
        )

    async def set_contact_representative(
        self, verification_id: VerificationId, representative: ContactRepresentative
    ) -> None:
        """Set contact representative.

        Args:
            verification_id: Verification id.
            representative: Contact representative.

        Raises:
            VerificationDoesNotExist: If verification does not exist.
            IncorrectStatus: If verification is in incorrect state.
        """
        request = SetContactRepresentativeRequest()
        request.verification_id.value = verification_id
        request.organization = representative.organization
        request.address.street[:] = representative.address.street
        request.address.city = representative.address.city
        request.address.state_or_province = representative.address.state_or_province
        request.address.postal_code = representative.address.postal_code
        request.address.country_code = representative.address.country_code
        await self.call(self.service, "set_contact_representative", request)
